	 /**************************************************/
	 /*                                                */
	 /*       HOST485.C -  RS-485 Host Communication   */
	 /*                    with T44P controller        */
	 /*                                                */
	 /*            Copyright (c) 1992                  */
	 /*            Triangle Research                   */
	 /**************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>

int multipt_command(char *string,char *response);

void transmit485(void);
void receive485(void);
int serial_getc(void);
void serial_out(unsigned char ch);
void init_serial(void);
int serial_in();
int serial_status(); 
void comm_error(void);

int commPort,commStatus,commData, commControl, commLineControl;

int  count=0;


void main()
{
  char command[20], response[100];
  char ch;
  int i;

  printf("\nCOM port (1 or 2)? >");
  scanf("%d",&ch);
  if (ch==1)  commPort = 0;    /* commPort = 0 for COM1: */
  else commPort = 1;           /* commPort = 1 for COM2: */

  ch=getchar(); 	       /* clear keyboard buffer */
  init_serial();               /* initialize PC's serial port */
  clrscr();                    /* clear PC's screen */

  printf("   Send Point-to-Point/Multi-point command (with '@') to T44P\n");
  printf("   ==========================================================\n");

  while (1) {
    printf("Command String ? ");
    gets(command);
    if (send_command(command, response)==-1) comm_error();
    else printf("Response String: %s\n\n",response);
  }
}
/*-------------------------------------------------------------------------*/
/*  This function establish a Point-to-point or Multipoint communication   */
/*  with the target controller.                                            */
/*                                                                         */
/*  For point to point communication, it sends a Ctrl-E character and      */
/*  waits to obtain echo of Ctrl-E from the controller. For a multipoint   */
/*  command (string starts with '@' character) it will not send Ctrl-E     */
/*                                                                         */
/*  The command "string" is subsequently sent to the controller via the    */
/*  serial port. It will then wait to receive the response from the        */
/*  controller and store it in the "response" string.                      */
/*                                                                         */
/*  Function return (-1) if communication error occurs                     */
/*-------------------------------------------------------------------------*/

int send_command(char *string,char *response)
{
    int i;
    int ch;

    if (commStatus==0) init_serial();
    if (string[0]!= '@') {           /* its not a multipoint command */
      transmit485();
      serial_out(5);                 /* send Ctrl-E (ASCII number 5) */
      receive485();

      if ((ch=serial_getc()) < 0)    /* error receive  echo */
	 return (-1);
    }
    transmit485();
    for (i=0;string[i]!=0;i++) serial_out(string[i]);
    serial_out('\x0D');
    receive485();

    for (i=0;i<100;i++)              /* wait to receive response */
    {
       if ((ch=serial_getc()) == -1)
	  return (-1);
       response[i]=ch;
       if (ch=='*') {
	  ch=serial_getc();
	  response[i+1]=0;
	  break;
       }
    }
    return (1);
}

/*-------------------------------------------------------------------------*/
/*  The following two functions enables/disables the RS485 transmitter     */
/*  connected to the host computer. These codes are written for an         */
/*  RS232-to-RS485 converter where the transmitter is controlled by the    */
/*  /RTS pin of the host serial port.                                      */
/*  Users may need to modify this function to suit the type of RS485 	   */
/*  adapter used.                                                     	   */
/*-------------------------------------------------------------------------*/

void transmit485(void)
{
  outportb(commControl,3);   /* Turn on /RTS to enable RS485 transmitter */
}                            /* /DTR is enabled for Opto-232 supply    */

void receive485(void)
{
  while (!(inportb(commStatus) & 0x40)); /* Make sure no more characters */
				      /* in the transmit shift register  */
  outportb(commControl,1);   /*  Turn off /RTS to enable RS485 receiver  */
}

/*-------------------------------------------------------------------------*/
/* This function check the serial port "commPort" (COM1 = 0; COM2 = 1) for */
/* data. It returns the 7-bit data if ready.                               */
/* If after 2000 tries still no data ready, function return (-1) to        */
/* indicate time-out error. This is to prevent locking up the computer in  */
/* a dead loop if device does not respond.                                 */
/*-------------------------------------------------------------------------*/

int serial_getc(void)
{
   int i,j;
   for (j=0;j<10;j++) {
     for (i=0;i<2000;i++) {                 /* allow for time out */
	if (serial_status())
	    return (serial_in());
     }
   }
   return(-1);                           /* nothing received, time-out !! */
}
/*-------------------------------------------------------------------------*/
/* Output a character to the serial port                                   */
/*-------------------------------------------------------------------------*/

void serial_out(unsigned char ch) {
  outportb(commData,ch);
  while (!(inportb(commStatus) & 0x20))
       ;
}
/*-------------------------------------------------------------------------*/
/*  Initialize the IBMPC serial communication port to 9600, 8 data bits,   */
/*  1 stop bit and no parity.                                              */
/*  must be called once before communicating with T44P                     */
/*-------------------------------------------------------------------------*/

#define SETTINGS (0x03|0x00|0x00|0xE0)

void init_serial(void)
{
   set_baud_rate(3);       /* initialize to 9600 baud, 8 data, 1 stop, no parity */
}
/*--------------------------------------------------------------------------*/
/* The baud rate of the PC serial port can be changed by the following      */
/* function using a divisor to divide the baud rate. The divisor value for  */
/* each supported baud rate is contained in the array "divisor[]".          */
/*    The input argument "baud_select" determines the baud rate as follow:  */
/*         Baud rate        baud_select                                     */
/*         =========        ===========                                     */
/*          1200               0                                            */
/*          2400               1                                            */
/*          4800               2                                            */
/*          9600               3                                            */
/*          19200              4                                            */
/*          38400              5                                            */
/*--------------------------------------------------------------------------*/

int divisor[]={ 96,48,24,12,6,3 };

set_baud_rate(int baud_select)
{
   if (commPort==0)
   {
      commStatus = 0x3FD;
      commData   = 0x3F8;
      commControl= 0x3FC;
      commLineControl = 0x3FB;
   }
   else
   {
      commStatus = 0x2FD;
      commData   = 0x2F8;
      commControl= 0x2FC;
      commLineControl = 0x2FB;
   }
   outportb(commLineControl,inportb(commLineControl) | 0x80);  /* set DLAB */
   outportb(commData, divisor[baud_select]);
   outportb(commData+1, 0);
   outportb(commLineControl,inportb(commLineControl) & 0x7F);  /* clear DLAB*/

   outportb(commControl,1);    /* Force /DTR to low for Opto232 use */
   while (serial_in());        /* Flush all characters in serial buffer */
}
/*-------------------------------------------------------------------------*/
/*  If a character is ready at the 8250 UART, return the character.        */
/*  Otherwise return 0.                                                    */
/*-------------------------------------------------------------------------*/

int serial_in() {
  if (inportb(commStatus) & 0x01) return(inportb(commData));
  return(0);
}
/*-------------------------------------------------------------------------*/
/*  Check 8250 UART status.                                                */
/*    returns 1 if a character is present, otherwise returns 0             */
/*-------------------------------------------------------------------------*/

int serial_status() {
  char ch;
  ch = inportb(commStatus);
  if ((ch & 0x0F) != 0)
     return (-1);
  return (inportb(commStatus) & 0x01);
}
/*-------------------------------------------------------------------------*/
/*  Report to user that a communication error occur between the computer   */
/*  and the controller                                                     */
/*-------------------------------------------------------------------------*/

void comm_error(void)
{
  int k;
  printf(
    "\n\tCommunication Error!... Check serial port COM%d:\n\n",commPort+1);
}
/*-------------------------------------------------------------------------*/

